@@ -10,7 +10,7 @@ from sqlalchemy.orm import sessionmaker, scoped_session
 
 
 class DAO(object):
-    session = None
+    # session = None
 
     def __init__(self):
         host_server = os.environ.get('POSTGRES_HOST', 'postgres')
@@ -29,36 +29,47 @@ class DAO(object):
         # database = databases.Database(DATABASE_URL)
 
         # Session = sessionmaker(bind=engine)
+        # session_factory = sessionmaker(bind=self.engine)
+        # Session = scoped_session(session_factory)
+        # self.session = Session()
 
-    def save_object(self, obj):
-        # try:
-        self.session.add(obj)
-        self.session.commit()
-        # except IntegrityError:
-        #     print('error')
-        #     self.session.rollback()
+    @staticmethod
+    def save_object(session, obj):
+        try:
+            session.add(obj)
+            session.commit()
+        except IntegrityError:
+            print('IntegrityError')
+            session.rollback()
 
-    def get_object(self, table, key):
-        result = self.session.query(table).filter_by(**key).first()
+    @staticmethod
+    def get_object(session, table, key):
+        result = session.query(table).filter_by(**key).first()
         if not result:
             return None
         return result
 
-    def save_if_not_exist(self, obj, table, kwargs):
-        obj_db = self.get_object(table, kwargs)
+    @staticmethod
+    def save_if_not_exist(session, obj, table, kwargs):
+        obj_db = DAO.get_object(session, table, kwargs)
         if obj_db:
             return obj_db
 
-        self.save_object(obj)
+        DAO.save_object(session, obj)
         return obj
 
     def get_publication(self, doi):
-        return self.get_object(Publication, {'doi': doi})
+        session_factory = sessionmaker(bind=self.engine)
+        Session = scoped_session(session_factory)
+        session = Session()
+        result = self.get_object(session, Publication, {'doi': doi})
+        session.close()
+        return result
 
     def save_publication(self, publication_data):
         session_factory = sessionmaker(bind=self.engine)
         Session = scoped_session(session_factory)
-        self.session = Session()
+        session = Session()
 
         publication = Publication(doi=publication_data['doi'], type=publication_data['type'],
                                   pubDate=publication_data['pubDate'], year=publication_data['year'],
@@ -67,7 +78,7 @@ class DAO(object):
                                   title=publication_data['title'],
                                   normalizedTitle=publication_data['normalizedTitle'],
                                   abstract=publication_data['abstract'])
-        publication = self.save_if_not_exist(publication, Publication, {'doi': publication.doi})
+        publication = self.save_if_not_exist(session, publication, Publication, {'doi': publication.doi})
 
         logging.warning('publication.doi')
         logging.warning(publication.doi)
@@ -77,37 +88,38 @@ class DAO(object):
         for author_data in authors:
             author = Author(name=author_data['name'], normalizedName=author_data['normalizedName'])
 
-            author = self.save_if_not_exist(author, Author, {'normalizedName': author.normalizedName})
+            author = self.save_if_not_exist(session, author, Author, {'normalizedName': author.normalizedName})
             logging.warning('author.id')
             logging.warning(author.id)
             if author.id:
                 publication_authors = PublicationAuthor(**{'authorId': author.id, 'publicationDoi': publication.doi})
-                self.save_if_not_exist(publication_authors, PublicationAuthor, {'authorId': author.id, 'publicationDoi': publication.doi})
+                self.save_if_not_exist(session, publication_authors, PublicationAuthor, {'authorId': author.id, 'publicationDoi': publication.doi})
 
         sources = publication_data['source_id']
         for sources_data in sources:
             source = Source(title=sources_data['title'], url=sources_data['url']) # todo no doi url ?
-            source = self.save_if_not_exist(source, Source, {'title': source.title})
+            source = self.save_if_not_exist(session, source, Source, {'title': source.title})
             logging.warning('source.id')
             logging.warning(source.id)
             if source.id:
                 publication_sources = PublicationSource(**{'sourceId': source.id, 'publicationDoi': publication.doi})
-                self.save_if_not_exist(publication_sources, PublicationSource, {'sourceId': source.id, 'publicationDoi': publication.doi})
+                self.save_if_not_exist(session, publication_sources, PublicationSource, {'sourceId': source.id, 'publicationDoi': publication.doi})
 
         if 'fieldsOfStudy' in publication_data:
             fields_of_study = publication_data['fieldsOfStudy']
             for fos_data in fields_of_study:
                 fos = FieldOfStudy(name=fos_data['name'], normalizedName=fos_data['normalizedName'])
                 fos.level = 2
-                fos = self.save_if_not_exist(fos, FieldOfStudy, {'normalizedName': fos.normalizedName})
+                fos = self.save_if_not_exist(session, fos, FieldOfStudy, {'normalizedName': fos.normalizedName})
                 logging.warning('fos.id')
                 logging.warning(fos.id)
                 if fos.id:
                     publication_fos = PublicationFieldOfStudy(**{'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
-                    self.save_if_not_exist(publication_fos, PublicationFieldOfStudy, {'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
+                    self.save_if_not_exist(session, publication_fos, PublicationFieldOfStudy, {'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
 
-        self.session.close()
+        session.close()
         return publication
-        # todo
+        # todo add perculator!!!!!!
+        # use different names for config until we remove gql?
         # publicationCitations = PublicationCitations()
         # publicationReferences = PublicationReferences(**author_data)
